﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SimpleCalculator
{
    public partial class CalcWinForm : Form, SimpleCalculator.ICalcView
    {
        #region button clicked events
        // init all with empty delegates to avoid having to check for null when calling

        /// <summary>Occurs when a number button is clicked</summary>
        public event Action<int> NumberButtonClicked = delegate { };

        /// <summary>Occurs when an operator button is clicked</summary>
        public event Action<CalcEnums.Operators> OperatorButtonClicked = delegate { };
        
        /// <summary>Occurs when the decimal button is clicked</summary>
        public event Action DecimalButtonClicked = delegate { };

        /// <summary>Occurs when the clear button is clicked</summary>
        public event Action ClearButtonClicked = delegate { };

        /// <summary>Occurs when the = button is clicked</summary>
        public event Action EqualsButtonClicked = delegate { };
        #endregion

        private ICalcController controller;

        public CalcWinForm()
        {
            InitializeComponent();
            controller = new CalcController(new CalcModel(), this);
        }

        #region number button clicked
        /// <summary>0 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num0Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(0);
        }

        /// <summary>1 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num1Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(1);
        }

        /// <summary>2 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num2Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(2);
        }

        /// <summary>3 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num3Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(3);
        }

        /// <summary>4 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num4Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(4);
        }

        /// <summary>5 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num5Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(5);
        }

        /// <summary>6 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num6Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(6);
        }

        /// <summary>7 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num7Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(7);
        }

        /// <summary>8 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num8Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(8);
        }

        /// <summary>9 button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void num9Btn_Click(object sender, EventArgs e)
        {
            NumberButtonClicked(9);
        }
        #endregion

        #region operator button clicked
        /// <summary>+ button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void opAddBtn_Click(object sender, EventArgs e)
        {
            OperatorButtonClicked(CalcEnums.Operators.Add);
        }

        /// <summary>- button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void opSubBtn_Click(object sender, EventArgs e)
        {
            OperatorButtonClicked(CalcEnums.Operators.Subtract);
        }

        /// <summary>* button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void opMultBtn_Click(object sender, EventArgs e)
        {
            OperatorButtonClicked(CalcEnums.Operators.Multiply);
        }

        /// <summary>/ button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void opDivBtn_Click(object sender, EventArgs e)
        {
            OperatorButtonClicked(CalcEnums.Operators.Divide);
        }
        #endregion

        #region other button clicked
        /// <summary>Decimal point button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void decPtBtn_Click(object sender, EventArgs e)
        {
            DecimalButtonClicked();
        }

        /// <summary>Clear button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void clearBtn_Click(object sender, EventArgs e)
        {
            ClearButtonClicked();
        }

        /// <summary>= button clicked</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void eqBtn_Click(object sender, EventArgs e)
        {
            EqualsButtonClicked();   
        }
        #endregion

        #region enable buttons
        /// <summary>Enable/disable the number buttons</summary>
        /// <param name="enable">True to enable. Else disable</param>
        public void EnableNumberButtons(bool enable)
        {
            num0Btn.Enabled = enable;
            num1Btn.Enabled = enable;
            num2Btn.Enabled = enable;
            num3Btn.Enabled = enable;
            num4Btn.Enabled = enable;
            num5Btn.Enabled = enable;
            num6Btn.Enabled = enable;
            num7Btn.Enabled = enable;
            num8Btn.Enabled = enable;
            num9Btn.Enabled = enable;
        }

        /// <summary>Enable/disable the operator buttons</summary>
        /// <param name="enable">True to enable. Else disable</param>
        public void EnableOpButtons(bool enable)
        {
            opAddBtn.Enabled = enable;
            opSubBtn.Enabled = enable;
            opMultBtn.Enabled = enable;
            opDivBtn.Enabled = enable;
        }

        /// <summary>Enable/disable the = button</summary>
        /// <param name="enable">True to enable. Else disable</param>
        public void EnableEq(bool enable)
        {
            eqBtn.Enabled = enable;
        }
        #endregion

        /// <summary>Display text on calc screen</summary>
        /// <param name="dispStr">Text to display</param>
        public void Display(string dispStr)
        {
            outputTb.ForeColor = Color.Black;
            outputTb.Text = dispStr;
        }

        /// <summary>Display an error on the calc screen</summary>
        /// <param name="errorMsg">Error message to display</param>
        public void DisplayError(string errorMsg)
        {
            outputTb.ForeColor = Color.Red;
            outputTb.Text = errorMsg;
        }
    }
}
